#!/usr/bin/env perl
#
# find-deps - find DLLs required by (but not in) the DLLs in CWD
#
# Copyright (C) 2022 Corwin Brust
#
# Quick and dirty check for missing dependancies of the DLLs included
# with a recently packaged build of Emacs. This should work given
# building Emacs works.  Run it from the install folder used for
# packaging, e.g.:
#
#   cd ~/emacs-build/install/emacs-28.0.91/bin
#   perl ../../../find-deps.pl > ~/emacs-build/dep-search.txt
# Or:
#  perl ../../../find-deps.pl | xargs -I '{}' cp '{}' .
#
# this rather obtuse alternative catches a couple more:
#   find ./ -name '*.dll' -print0 | \
#    xargs -0 -I '{}' cygcheck '{}' 2>&1 | \
#    grep 'could not' | cut -d ' ' -f 6 | \
#    sort | uniq | xargs -I '{}' cp '{}' .

## This file is not part of GNU Emacs.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <https://www.gnu.org/licenses/>.

use strict;
use warnings qw[all];

use File::Basename;

my %deps;

# use ldd to find deps
sub deps_of(_) {
  local $_ = `ldd $_`;
  s/^\s+//msg;
  map [ split / => / ]->[1] || $_,
    map { s/ \(.*?\)//; $_ }
    grep !/not found/,  # ignore optional
    grep !m|/Windows/|i,# ignore */Windows/*
    split /\n/
}

# call deps_of for each arg (unless we already have)
sub walk_deps(@) {
  my $found_count = 0;
  for (@_) {
    my $key = fileparse($_);
    next if exists $deps{$key} or !/.dll$/;
    my @found = deps_of $_;
    if(@found) {
      $deps{$key} = \@found;
      $found_count += 1;
    }
  }
  return $found_count;
}

# get a list of DLLs to start with
my @libs = glob('*.dll');

# search for deps of DLLs we included
walk_deps @libs;

# search for deps of our deps
my $added_count = 0;
do {
  $added_count = walk_deps map @$_, values %deps
} while($added_count > 0);

# de-dup
my %uniq;
for(map @$_, values %deps) {
  my $dll = fileparse($_);
  next if -e $dll; # skip those we included
  $uniq{$_} += 1;
}

# report
$\ = "\n";
print for sort keys %uniq;
warn
  "searched: ",  scalar(@libs),
  ", found: ",   scalar( map @$_, values %deps),
  ", of which ", scalar(keys %uniq),
  " are missing in ", `pwd`;
